package AERPP;
import ilog.concert.IloException;
import ilog.concert.IloNumExpr;
import ilog.concert.IloNumVar;
import ilog.concert.IloRange;
import ilog.cplex.IloCplex;
import java.util.ArrayList;

public class L0MixL1Selector {

	private double[][] Pi;
	private int p;// number of inequalities from step 1
	private int noOfCause;
	private int noOfPostivePII;
	private double finalObjValue;
	private double runningTime;
	private double startingTime;
	private double endingTime;
	private ArrayList<int[]> AExact;
	private ArrayList<Double> cOb;
	private ArrayList<SelectedG> selectedGList;
	private final double eps=1e-8;
	private double error;
	private ArrayList<int[]> AError;
	private ArrayList<Double> cError;
	private ArrayList<Integer> L0List;
	private ArrayList<Integer> finalGList;;



	public L0MixL1Selector(ArrayList<int[]> AExact, ArrayList<Double> cOb, double error){
		startingTime=System.currentTimeMillis();
		this.AExact=AExact;
		this.cOb=cOb;
		this.noOfCause=AExact.get(0).length;
		this.p=AExact.size();
		this.error=error;
		selectedGList=new ArrayList<SelectedG>();
		this.AError=new ArrayList<int[]>();
		this.cError=new ArrayList<Double>();
		this.L0List=new ArrayList<Integer>();
		this.finalGList=new ArrayList<Integer>();
	}

	public void slovePIIG() throws IloException{
		//
		IloCplex cplex = new IloCplex();
		ArrayList<IloNumVar[]> pi=new ArrayList<IloNumVar[]>(p);
		IloNumVar[] g=new IloNumVar[p-1];
		IloNumVar[] g1=new IloNumVar[p-1];

		IloNumVar[] piN=new IloNumVar[p-1];

		for (int i=0;i<p;i++){
			ArrayList<Integer> piNumberList=new ArrayList<Integer>();
			int[] ai=AExact.get(i);
			for (int j=0;j<p;j++){
				int[] aj=AExact.get(j);
				boolean connect=false;
				for (int k=0;k<noOfCause;k++){
					if (ai[k]==aj[k]&&ai[k]==1){
						connect=true;
						break;
					}
				}
				if (connect==true){
					piNumberList.add(j);
				}
			}
			ArrayList<Integer> kList=new ArrayList<Integer>();
			for (int k=0;k<noOfCause;k++){
				if (ai[k]>0){
					kList.add(k);
				}
			}
			SelectedG sg=new SelectedG(i,piNumberList,kList,false);
			selectedGList.add(sg);
			//	System.out.println("i="+i+", piList:"+piNumberList);
		}

		//initiate Pi
		for (int i=0;i<p;i++){
			int noOfPi=selectedGList.get(i).getPiNumberList().size();
			IloNumVar[] pp=new IloNumVar[noOfPi];
			for (int j=0;j<noOfPi;j++){//
				pp[j]=cplex.numVar(0, 5);
			}
			pi.add(pp);// pi.get(j)[i] equals to pi[i][j]
		}
		
		for (int i=0;i<p-1;i++){
			g[i]=cplex.boolVar();
			g1[i]=cplex.numVar(0, 5);
			piN[i]=cplex.numVar(0, 15);
		}

		// pi*A>=A, for all Ai
		IloRange[][] piAARange=new IloRange[p][noOfCause];	
		IloNumExpr[][] piAAExp=new IloNumExpr[p][noOfCause];
		for (int i=0;i<p-1;i++){
			int[] ai=AExact.get(i);
			for (int k=0;k<noOfCause;k++){
				IloNumExpr piA=cplex.numExpr();						
				for (int j=0;j<p;j++){
					ArrayList<Integer> piNumList=selectedGList.get(j).getPiNumberList();
					int[] aj=AExact.get(j);
					for (int check=0;check<piNumList.size();check++){
						if (piNumList.get(check)==i){
							piA=cplex.sum(piA,cplex.prod(pi.get(j)[check],aj[k]));	
							break;
						}
					}
				}
				piA=cplex.sum(piA,cplex.prod(-1, piN[i]));
				IloNumExpr left=cplex.sum(piA,-ai[k]);
				piAARange[i][k]=cplex.addGe(left, 0);
				piAAExp[i][k]=left;
			}
		}

		// (pi*c-c)<=v, for all ci
		IloRange[] piCCRange=new IloRange[p];
		IloNumExpr[] piCCExp=new IloNumExpr[p];
		for (int i=0;i<p-1;i++){
			IloNumExpr piC=cplex.numExpr();			
			for (int j=0;j<p;j++){
				ArrayList<Integer> piNumList=selectedGList.get(j).getPiNumberList();
				for (int check=0;check<piNumList.size();check++){
					if (piNumList.get(check)==i){
						piC=cplex.sum(piC,cplex.prod(pi.get(j)[check],cOb.get(j)));	
						break;
					}
				}
			}
			piC=cplex.sum(piC,cplex.prod(-1, piN[i]));
			IloNumExpr left=cplex.sum(piC,-cOb.get(i));
			left=cplex.sum(left,-error);
			piCCExp[i]=left;
			piCCRange[i]=cplex.addLe(left, 0);
		}
		for (int i=0;i<p-1;i++){
			for (int j=0;j<pi.get(i).length;j++){
				cplex.addLe(pi.get(i)[j],cplex.prod(5, g[i]));
				cplex.addLe(pi.get(i)[j],g1[i]);
			}
		}
		// obj
		IloNumExpr obj=cplex.numExpr();
		for (int i=0;i<p-1;i++){//
			obj=cplex.sum(obj,g[i]);
			obj=cplex.sum(obj,cplex.prod(0.0001, g1[i]));
		}
		for (int i=0;i<noOfCause;i++){
		//	System.out.print(AExact.get(p-1)[i]+",");
		}

		cplex.addMinimize(obj);
		if (cplex.solve()){		
			finalObjValue=cplex.getObjValue();
			for (int i=0;i<p-1;i++){// Warning! Should back to check here!		
				if (cplex.getValue(g[i])==1){
					L0List.add(i);
					AError.add(AExact.get(i));
					cError.add(cOb.get(i));
					System.out.println("G "+i+"="+cplex.getValue(g[i]));
					noOfPostivePII++;
				}
			}
			AError.add(AExact.get(p-1));
			cError.add(cOb.get(p-1));
			noOfPostivePII++;
			
			for (int i=0;i<p-1;i++){
				finalGList.add((int) cplex.getValue(g[i]));
			}
			for (int i=0;i<p-1;i++){
				if (cplex.getValue(piN[i])>eps){
					System.out.println("piN["+i+"]="+cplex.getValue(piN[i]));
				//	System.exit(0);
				}
			}
			System.out.println("Optimal Objective Value (L0 Selector):"+cplex.getObjValue());
			System.out.println("Number of Positive G final (L0 Selector):"+noOfPostivePII+" from "+selectedGList.size()+" from "+p);
		}
		cplex.end();
		endingTime=System.currentTimeMillis();
		runningTime=(endingTime-startingTime)/1000;
	}

	public double getFinalObjValue() {
		return finalObjValue;
	}

	public double getRunningTime() {
		return runningTime;
	}

	public double[][] getPi() {
		return Pi;
	}

	public ArrayList<int[]> getAError() {
		return AError;
	}

	public ArrayList<Double> getcError() {
		return cError;
	}

	public ArrayList<Integer> getL0List() {
		return L0List;
	}

	public ArrayList<Integer> getFinalGList() {
		return finalGList;
	}


	

}
