package AERPP;
import ilog.concert.IloException;
import ilog.concert.IloNumExpr;
import ilog.concert.IloNumVar;
import ilog.concert.IloRange;
import ilog.cplex.IloCplex;
import java.util.ArrayList;

public class L1Selector {

	private double[][] Pi;
	private int p;// number of inequalities from step 1
	private int noOfCause;
	private int noOfPostivePII;
	private int noOfPostiveGCutoff;
	private int noOfPostiveGCutoff2;
	private double finalObjValue;
	private double runningTime;
	private double startingTime;
	private double endingTime;
	private ArrayList<int[]> AExact;
	private ArrayList<Double> cOb;
	private ArrayList<SelectedG> selectedGList;
	private final double eps=1e-6;
	private double error;
	private ArrayList<int[]> AError;
	private ArrayList<Double> cError;
	private ArrayList<int[]> ACutoff;
	private ArrayList<Double> cCutoff;
	private ArrayList<int[]> ACutoff2;
	private ArrayList<Double> cCutoff2;
	private ArrayList<Double> finalGList;
	private ArrayList<Integer> L1List;
	private double cutoff;
	private double cutoff2;



	public L1Selector(ArrayList<int[]> AExact, ArrayList<Double> cOb, double error, double cutoff, double cutoff2){
		startingTime=System.currentTimeMillis();
		this.AExact=AExact;
		this.cOb=cOb;
		this.noOfCause=AExact.get(0).length;
		this.p=AExact.size();
		this.error=error;
		selectedGList=new ArrayList<SelectedG>();
		this.AError=new ArrayList<int[]>();
		this.cError=new ArrayList<Double>();
		this.ACutoff=new ArrayList<int[]>();
		this.cCutoff=new ArrayList<Double>();
		this.ACutoff2=new ArrayList<int[]>();
		this.cCutoff2=new ArrayList<Double>();
		this.finalGList=new ArrayList<Double>();
		this.cutoff=cutoff;
		this.cutoff2=cutoff2;
		this.L1List=new ArrayList<Integer>();

	}


	public void slovePIIG() throws IloException{
		//
		IloCplex cplex = new IloCplex();
		ArrayList<IloNumVar[]> pi=new ArrayList<IloNumVar[]>(p);
		IloNumVar[] g=new IloNumVar[p-1];
		IloNumVar[] piN=new IloNumVar[p-1];

		for (int i=0;i<p;i++){
			ArrayList<Integer> piNumberList=new ArrayList<Integer>();
			int[] ai=AExact.get(i);
			for (int j=0;j<p;j++){
				int[] aj=AExact.get(j);
				boolean connect=false;
				for (int k=0;k<noOfCause;k++){
					if (ai[k]==aj[k]&&ai[k]==1){
						connect=true;
						break;
					}
				}
				if (connect==true){
					piNumberList.add(j);
				}
			}
			ArrayList<Integer> kList=new ArrayList<Integer>();
			for (int k=0;k<noOfCause;k++){
				if (ai[k]>0){
					kList.add(k);
				}
			}
			SelectedG sg=new SelectedG(i,piNumberList,kList,false);
			selectedGList.add(sg);
			//	System.out.println("i="+i+", piList:"+piNumberList);
		}

		//initiate Pi
		for (int i=0;i<p;i++){
			int noOfPi=selectedGList.get(i).getPiNumberList().size();
			IloNumVar[] pp=new IloNumVar[noOfPi];
			for (int j=0;j<noOfPi;j++){//
				pp[j]=cplex.numVar(0, 5);
			}
			pi.add(pp);// pi.get(j)[i] equals to pi[i][j]
		}

		for (int i=0;i<p-1;i++){
			g[i]=cplex.numVar(0, 5);
			piN[i]=cplex.numVar(0, 15);
		}

		// pi*A>=A, for all Ai
		//	IloRange[][] piAARange=new IloRange[p][noOfCause];	
		//	IloNumExpr[][] piAAExp=new IloNumExpr[p][noOfCause];
		for (int i=0;i<p-1;i++){
			int[] ai=AExact.get(i);
			for (int k=0;k<noOfCause;k++){
				IloNumExpr piA=cplex.numExpr();						
				for (int j=0;j<p;j++){
					ArrayList<Integer> piNumList=selectedGList.get(j).getPiNumberList();
					int[] aj=AExact.get(j);
					for (int check=0;check<piNumList.size();check++){
						if (piNumList.get(check)==i){
							piA=cplex.sum(piA,cplex.prod(pi.get(j)[check],aj[k]));	
							break;
						}
					}
				}
				piA=cplex.sum(piA,cplex.prod(-1, piN[i]));
				IloNumExpr left=cplex.sum(piA,-ai[k]);
				cplex.addGe(left, 0);
				//		piAARange[i][k]=cplex.addGe(left, 0);
				//		piAAExp[i][k]=left;
			}
		}

		// (pi*c-c)<=v, for all ci
		//	IloRange[] piCCRange=new IloRange[p];
		//	IloNumExpr[] piCCExp=new IloNumExpr[p];
		for (int i=0;i<p-1;i++){
			IloNumExpr piC=cplex.numExpr();			
			for (int j=0;j<p;j++){
				ArrayList<Integer> piNumList=selectedGList.get(j).getPiNumberList();
				for (int check=0;check<piNumList.size();check++){
					if (piNumList.get(check)==i){
						piC=cplex.sum(piC,cplex.prod(pi.get(j)[check],cOb.get(j)));	
						break;
					}
				}
			}
			piC=cplex.sum(piC,cplex.prod(-1, piN[i]));
			IloNumExpr left=cplex.sum(piC,-cOb.get(i));
			left=cplex.sum(left,-error);
			cplex.addLe(left, 0);
			//	piCCExp[i]=left;
			//	piCCRange[i]=cplex.addLe(left, 0);
		}
		for (int i=0;i<p-1;i++){
			for (int j=0;j<pi.get(i).length;j++){
				cplex.addLe(pi.get(i)[j],g[i]);
			}
		}
		// obj
		IloNumExpr obj=cplex.numExpr();
		for (int i=0;i<p-1;i++){//
			obj=cplex.sum(obj,g[i]);
		}
		for (int i=0;i<noOfCause;i++){
			//	System.out.print(AExact.get(p-1)[i]+",");
		}

		cplex.addMinimize(obj);
		if (cplex.solve()){		
			finalObjValue=cplex.getObjValue();
			for (int i=0;i<p-1;i++){
				finalGList.add(cplex.getValue(g[i]));
			}		
			for (int i=0;i<p-1;i++){// Warning! Should back to check here!		
				if (cplex.getValue(g[i])>eps){
					AError.add(AExact.get(i));
					cError.add(cOb.get(i));
					System.out.println("G "+i+"="+cplex.getValue(g[i]));
				//	for (int j=0;j<pi.get(i).length;j++){
				//		System.out.println("pi "+j+":"+cplex.getValue(pi.get(i)[j]));
				//	}
					noOfPostivePII++;
					L1List.add(i);
				}
				if (cplex.getValue(g[i])>cutoff){
					ACutoff.add(AExact.get(i));
					cCutoff.add(cOb.get(i));
					System.out.println("Cutoff G "+i+"="+cplex.getValue(g[i]));
					noOfPostiveGCutoff++;
				}	
				if (cplex.getValue(g[i])>cutoff2){
					ACutoff2.add(AExact.get(i));
					cCutoff2.add(cOb.get(i));
					System.out.println("Cutoff G2 "+i+"="+cplex.getValue(g[i]));
					noOfPostiveGCutoff2++;
				}
			}
			AError.add(AExact.get(p-1));
			cError.add(cOb.get(p-1));
			ACutoff.add(AExact.get(p-1));
			cCutoff.add(cOb.get(p-1));
			ACutoff2.add(AExact.get(p-1));
			cCutoff2.add(cOb.get(p-1));
			noOfPostiveGCutoff++;
			noOfPostivePII++;
			noOfPostiveGCutoff2++;
			System.out.println("Optimal Objective Value:"+cplex.getObjValue());
			System.out.println("Number of Positive G final:"+noOfPostivePII+" from "+selectedGList.size()+" from "+p);
			System.out.println("Number of Positive G Cutoff:"+noOfPostiveGCutoff+" from "+selectedGList.size()+" from "+p);
			System.out.println("Number of Positive G 2 Cutoff:"+noOfPostiveGCutoff2+" from "+selectedGList.size()+" from "+p);


		}
		cplex.end();
		endingTime=System.currentTimeMillis();
		runningTime=(endingTime-startingTime)/1000;
	}

	public double getFinalObjValue() {
		return finalObjValue;
	}

	public double getRunningTime() {
		return runningTime;
	}

	public double[][] getPi() {
		return Pi;
	}

	public ArrayList<int[]> getAError() {
		return AError;
	}

	public ArrayList<Double> getcError() {
		return cError;
	}

	public ArrayList<Double> getFinalGList() {
		return finalGList;
	}


	public ArrayList<int[]> getACutoff() {
		return ACutoff;
	}

	public ArrayList<Double> getcCutoff() {
		return cCutoff;
	}


	public ArrayList<int[]> getACutoff2() {
		return ACutoff2;
	}


	public ArrayList<Double> getcCutoff2() {
		return cCutoff2;
	}


	public ArrayList<Integer> getL1List() {
		return L1List;
	}



}
