package AERPP;

import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

import ilog.concert.IloException;

public class MonteCarloExperiment {

	/**
	 * @param args
	 * @throws IloException 
	 * @throws IOException 
	 */
	public static void main(String[] args) throws IloException, IOException {
		// TODO Auto-generated method stub
		long st=System.currentTimeMillis();
	
		double[] realP=new double[25];
		int[][] networkIndicator=new int[15][25];

		// generate miu(j)
		double tot=0;
		for (int i=1;i<=25;i++){
			tot+=Math.pow(Math.abs(i-13),1.5);
			if (i==13){
				tot+=Math.pow(1,1.5);
			}
		}
		System.out.println("Real P:");
		for (int i=1;i<=25;i++){
			realP[i-1]=Math.pow(Math.abs(i-13),1.5)/tot;
			if (i==13){
				realP[i-1]=Math.pow(1,1.5)/tot;
			}
			System.out.println("P["+i+"]="+realP[i-1]+",");
		}
		System.out.println();
		// generate phi
		System.out.println("Adjacent Matrix:");
		for (int i=1;i<=15;i++){
			networkIndicator[i-1]=new int[25];
			if (i==1){
				networkIndicator[i-1][i-1]=1;
				networkIndicator[i-1][i]=1;
				networkIndicator[i-1][i-1+10-1]=1;
				networkIndicator[i-1][i-1+10]=1;
				networkIndicator[i-1][i-1+10+1]=1;
			}
			else if(i==15){
				networkIndicator[i-1][i-1-1]=1;
				networkIndicator[i-1][i-1]=1;
				networkIndicator[i-1][i-1+1]=1;
				networkIndicator[i-1][i-1+10-1]=1;
				networkIndicator[i-1][i-1+10]=1;
			}
			else{
				networkIndicator[i-1][i-1-1]=1;
				networkIndicator[i-1][i-1]=1;
				networkIndicator[i-1][i-1+1]=1;
				networkIndicator[i-1][i-1+10-1]=1;
				networkIndicator[i-1][i-1+10]=1;
				networkIndicator[i-1][i-1+10+1]=1;
			}
			for (int j=0;j<25;j++){
				System.out.print(networkIndicator[i-1][j]+",");
			}
			System.out.println();
		}
		
		boolean pii=false;// select the g(pii) format
		int noOfCause=networkIndicator.length;
		int noOfOb=realP.length;
		int numberOfExperiments=1;//M
		String name="Result";
		int numberOfCasesInEachExperiment=20000;//N
		double cutoff=0.1;
		double cutoff2=0.2;

		OneExperiment Ep=new OneExperiment(realP,networkIndicator,0,pii,cutoff, cutoff2);
		Ep.solveExact();
	//	System.exit(0);
		int p=Ep.getAReal().size();
		System.out.println("Total number of inequalities in exact description = "+p);

		GenerateRandomObservation GRO=new GenerateRandomObservation(realP,numberOfExperiments,numberOfCasesInEachExperiment);
		GRO.generateObservationList();
		ArrayList<double[]> observeFList=GRO.getObservedFList();

		// Exact method:
		ArrayList<int[]> AReal=Ep.getAReal();

		int countCover=0;
		int countNotCover=0;
		int totNoInApp=0;
		int totNoCutoff=0;
		int totNoCutoff2=0;

		int cutoffCover=0;
		int cutoffNotCover=0;

		int cutoffCover2=0;
		int cutoffNotCover2=0;

		double avgLambda=0;
		double totLambda=0;
		double totObj=0;
		double avgObj=0;


		ArrayList<ArrayList<Double>> gList=new ArrayList<ArrayList<Double>>();

		for (int i=0;i<observeFList.size();i++){
			System.out.println("----------------------------Experiment "+i+" / "+numberOfExperiments+"-------------------------------");
			double[] observedF=observeFList.get(i);			
			double sigmaHatSquare=0;
			for (int j=0;j<noOfOb;j++){
				sigmaHatSquare=Math.max(sigmaHatSquare,observedF[j]-observedF[j]*observedF[j]);
			}
			double parameter=1.05;
			double alpha=0.05;
			double sigmaHat=Math.sqrt(sigmaHatSquare);
			System.out.println("sigmaHat="+sigmaHat);
			double lambda1=parameter*sigmaHat*Math.sqrt(networkIndicator.length-1)*Math.sqrt(Math.log(p/alpha)/Math.log(2.72)/numberOfCasesInEachExperiment);
			double lambda2=parameter/2*Math.sqrt(Math.log(p/alpha)/Math.log(2.72)/numberOfCasesInEachExperiment);
			double lambda=Math.min(lambda1,lambda2);
			lambda=lambda/2;
			totLambda+=lambda;
			
			Ep.setError(lambda);		
			Ep.setError(0.01);		

			ArrayList<Double> cObserved=new ArrayList<Double>();
			for (int ii=0;ii<AReal.size();ii++){
				int[] A=AReal.get(ii);
				double cOb=0;
				for (int k=0;k<noOfOb;k++){
					for (int j=0;j<noOfCause;j++){
						if (A[j]==1&&networkIndicator[j][k]==1){
							cOb+=observedF[k];
							break;
						}
					}
				}
				cObserved.add(cOb);
			}

			Ep.solveApp(cObserved);
			Ep.compareExactAndError();
			Ep.compareExactAndCutoff();
			Ep.compareExactAndCutoff2();
			totObj=Ep.getObj();
			gList.add(Ep.getL1Glsit());

			totNoInApp+=Ep.getAError().size();
			totNoCutoff+=Ep.getACutoff().size();
			totNoCutoff2+=Ep.getACutoff2().size();

			if (Ep.isErrorCoverExact()==true){
				countCover++;
			}
			else{
				countNotCover++;
			}
			if (Ep.isErrorCoverCutoff()==false){
				cutoffNotCover++;
			}
			else{
				cutoffCover++;
			}
			if (Ep.isErrorCoverCutoff2()==false){
				cutoffNotCover2++;
			}
			else{
				cutoffCover2++;
			}
		}

		avgLambda=totLambda/numberOfExperiments;
		avgObj=totObj/numberOfExperiments;
		double coverProbability=(double)(countCover)/(double)(numberOfExperiments);
		double coverProbabilityCutoff=(double)(cutoffCover)/(double)(numberOfExperiments);
		double coverProbabilityCutoff2=(double)(cutoffCover2)/(double)(numberOfExperiments);

		FileWriter fwOUT = new FileWriter("C:\\Users\\haiwang\\Dropbox (Personal)\\Office Synchronize\\"
				+ "Active Research\\8. Big Data\\Core Determining Class\\Monte Carlo\\"+name+".csv");
		for (int i=0;i<numberOfExperiments;i++){
			fwOUT.write("Experiment "+i+",");
		}
		fwOUT.write("\n");

		for (int j=0;j<gList.get(0).size();j++){
			for (int i=0;i<numberOfExperiments;i++){
				fwOUT.write(gList.get(i).get(j)+",");
			}
			fwOUT.write("\n");
		}

		fwOUT.close();


		System.out.println("-----------------Summary of the Experiments--------------------");
		System.out.println(networkIndicator.length+" x "+realP.length+" network");
		System.out.println("Number of experiments (M):"+numberOfExperiments);
		System.out.println("Number of observations (N):"+numberOfCasesInEachExperiment);
		System.out.println("Avg Lambda = "+avgLambda);
		System.out.println("-----");
		System.out.println("Cover:                  "+countCover);
		System.out.println("Not Cover:              "+countNotCover);
		System.out.println("Cutoff Cover:           "+cutoffCover);
		System.out.println("Cutoff, Not Cover:      "+cutoffNotCover);
		System.out.println("Cutoff 2 Cover:         "+cutoffCover2);
		System.out.println("Cutoff 2, Not Cover:    "+cutoffNotCover2);		
		System.out.println("-----");
		System.out.println("Number of constraints from exact method:"+AReal.size());
		System.out.println("Avg obj:"+avgObj);
		System.out.println("Avg Number of constraint from approximated method:"+(double)(totNoInApp)/(double)(numberOfExperiments));
		System.out.println("Avg Number of constraint from approximated method cutoff:"+(double)(totNoCutoff)/(double)(numberOfExperiments));
		System.out.println("Avg Number of constraint from approximated method cutoff 2:"+(double)(totNoCutoff2)/(double)(numberOfExperiments));
		System.out.println("Frequency that AError cover AExact = "+coverProbability);
		System.out.println("Frequency that AError cover AExact Cutoff= "+coverProbabilityCutoff);
		System.out.println("Frequency that AError cover AExact Cutoff= "+coverProbabilityCutoff2);
		System.out.println("Running time(min):"+(double)(System.currentTimeMillis()-st)/(double)1000/(double)60);
	}

}
