package AERPP;

import ilog.concert.IloException;
import ilog.concert.IloNumExpr;
import ilog.concert.IloNumVar;
import ilog.cplex.IloCplex;
import java.util.ArrayList;

public class OneExperiment {

	private double[] realP;
	private int[][] networkIndicator;
	private double error;
	private ArrayList<int[]> AEntire; 
	private ArrayList<int[]> AReal;
	private ArrayList<int[]> AError;
	private ArrayList<int[]> ACutoff;
	private ArrayList<int[]> ACutoff2;
	private ArrayList<Double> cReal;
	private ArrayList<Double> cError;
	private ArrayList<Double> cCutoff;
	private ArrayList<Double> cCutoff2;
	private boolean pii;
	private boolean errorCoverExact;
	private boolean errorCoverCutoff;
	private boolean errorCoverCutoff2;
	private int noOfCause;
	private double eps=1e-8;
	private double obj=0;
	private ArrayList<Integer> L0Glsit;
	private ArrayList<Double> L1Glsit;
	private ArrayList<Integer> L1List;
	private ArrayList<Integer> L0List;
	
	private ArrayList<Integer> L0MixL1List;

	private double cutoff;
	private double cutoff2;


	public OneExperiment(double[] realP, int[][] networkIndicator, double error, boolean pii, double cutoff, double cutoff2) {
		super();
		this.realP = realP;
		this.networkIndicator = networkIndicator;
		this.error = error;
		this.AReal=new ArrayList<int[]>();
		this.AError=new ArrayList<int[]>();
		this.cReal=new ArrayList<Double>();
		this.cError=new ArrayList<Double>();
		this.ACutoff=new ArrayList<int[]>();
		this.cCutoff=new ArrayList<Double>();
		this.ACutoff2=new ArrayList<int[]>();
		this.cCutoff2=new ArrayList<Double>();
		this.cutoff2=cutoff2;
		this.pii=pii;
		this.cutoff=cutoff;
		this.noOfCause=networkIndicator.length;
		this.errorCoverExact=true;
		this.L0Glsit=new ArrayList<Integer>();
		this.L1Glsit=new ArrayList<Double>();
		this.L1List=new ArrayList<Integer>();
		this.L0List=new ArrayList<Integer>();
		this.L0MixL1List=new ArrayList<Integer>();

	}

	public void solveExact(){	
		SmartSelectionEQ smeq=new SmartSelectionEQ(realP,networkIndicator);
		smeq.selection();
		AReal=smeq.getAEq();
		cReal=smeq.getcEq();	
		System.out.println("Number of exact:"+AReal.size());
		smeq.printAC();
	}

	public void getEntire(){	
		SmartSelectionEQ smeq=new SmartSelectionEQ(realP,networkIndicator);
		AEntire=smeq.getAEq();		
	}
	
	public void solveApp(ArrayList<Double> cOb) throws IloException{
		L1Selector lp=new L1Selector(AReal,cOb,error,cutoff,cutoff2);
		if (pii==true){
		//	lp.slovePII();
			obj=lp.getFinalObjValue();
		}
		else{
			lp.slovePIIG();
			obj=lp.getFinalObjValue();
		}
		AError=lp.getAError();
		cError=lp.getcError();
		ACutoff=lp.getACutoff();
		cCutoff=lp.getcCutoff();
		ACutoff2=lp.getACutoff2();
		cCutoff2=lp.getcCutoff2();
		L1Glsit=lp.getFinalGList();
		L1List=lp.getL1List();
	}

	public void solveAppL0(ArrayList<Double> cOb) throws IloException{
		L0Selector lp=new L0Selector(AReal,cOb,error);
		lp.slovePIIG();
		AError=lp.getAError();
		cError=lp.getcError();
		L0Glsit=lp.getFinalGList();
		L0List=lp.getL0List();
	}
	
	public void solveAppL0MixL1(ArrayList<Double> cOb) throws IloException{
		L0MixL1Selector lp=new L0MixL1Selector(AReal,cOb,error);
		lp.slovePIIG();
		L0MixL1List=lp.getL0List();
	}

	public void compareExactAndError() throws IloException{
		
		errorCoverExact=true;
		IloCplex cplex = new IloCplex();		
		for (int i=0;i<AError.size();i++){
			int[] A=AError.get(i);
			double c=cError.get(i);
			cplex.clearModel();
		//	IloCplex cplex = new IloCplex();
			IloNumVar[] x=new IloNumVar[noOfCause];

			for (int j=0;j<noOfCause;j++){
				x[j]=cplex.numVar(0, 1);
			}
			// constraints
			for (int j=0;j<AReal.size();j++){
				int[] AR=AReal.get(j);
				double cR=cReal.get(j);
				IloNumExpr expr=cplex.numExpr();
				for (int k=0;k<noOfCause;k++){
					expr=cplex.sum(expr,cplex.prod(AR[k], x[k]));
				}
				cplex.addLe(expr, cR);
			}

			IloNumExpr all=cplex.numExpr();
			for (int k=0;k<noOfCause;k++){
				all=cplex.sum(all,x[k]);
			}
			cplex.addEq(all, 1);

			// obj
			IloNumExpr obj=cplex.numExpr();
			for (int k=0;k<noOfCause;k++){
				obj=cplex.sum(obj,cplex.prod(A[k], x[k]));
			}
			cplex.addMaximize(obj);

			if (cplex.solve()){
				if (cplex.getObjValue()>c+eps+error){
					System.out.println("obj="+cplex.getObjValue()+", c="+c);
					errorCoverExact=false;
					break;
				}
			}
			if (i==AError.size()-1){
				System.out.println("This is the last check! ("+AError.size()+")");
			}
			//cplex.end();
		}
		cplex.end();
	}
	
	
public void compareExactAndCutoff() throws IloException{
		
		errorCoverCutoff=true;
		IloCplex cplex = new IloCplex();		
		for (int i=0;i<ACutoff.size();i++){
			int[] A=ACutoff.get(i);
			double c=cCutoff.get(i);
			cplex.clearModel();
		//	IloCplex cplex = new IloCplex();
			IloNumVar[] x=new IloNumVar[noOfCause];

			for (int j=0;j<noOfCause;j++){
				x[j]=cplex.numVar(0, 1);
			}
			// constraints
			for (int j=0;j<AReal.size();j++){
				int[] AR=AReal.get(j);
				double cR=cReal.get(j);
				IloNumExpr expr=cplex.numExpr();
				for (int k=0;k<noOfCause;k++){
					expr=cplex.sum(expr,cplex.prod(AR[k], x[k]));
				}
				cplex.addLe(expr, cR);
			}

			IloNumExpr all=cplex.numExpr();
			for (int k=0;k<noOfCause;k++){
				all=cplex.sum(all,x[k]);
			}
			cplex.addEq(all, 1);

			// obj
			IloNumExpr obj=cplex.numExpr();
			for (int k=0;k<noOfCause;k++){
				obj=cplex.sum(obj,cplex.prod(A[k], x[k]));
			}
			cplex.addMaximize(obj);

			if (cplex.solve()){
				if (cplex.getObjValue()>c+eps+error){
					System.out.println("obj="+cplex.getObjValue()+", c="+c);
					errorCoverCutoff=false;
					break;
				}
			}
			if (i==ACutoff.size()-1){
				System.out.println("This is the last check! ("+ACutoff.size()+")");
			}
			//cplex.end();
		}
		cplex.end();
	}


public void compareExactAndCutoff2() throws IloException{
	
	errorCoverCutoff2=true;
	IloCplex cplex = new IloCplex();		
	for (int i=0;i<ACutoff2.size();i++){
		int[] A=ACutoff2.get(i);
		double c=cCutoff2.get(i);
		cplex.clearModel();
	//	IloCplex cplex = new IloCplex();
		IloNumVar[] x=new IloNumVar[noOfCause];

		for (int j=0;j<noOfCause;j++){
			x[j]=cplex.numVar(0, 1);
		}
		// constraints
		for (int j=0;j<AReal.size();j++){
			int[] AR=AReal.get(j);
			double cR=cReal.get(j);
			IloNumExpr expr=cplex.numExpr();
			for (int k=0;k<noOfCause;k++){
				expr=cplex.sum(expr,cplex.prod(AR[k], x[k]));
			}
			cplex.addLe(expr, cR);
		}

		IloNumExpr all=cplex.numExpr();
		for (int k=0;k<noOfCause;k++){
			all=cplex.sum(all,x[k]);
		}
		cplex.addEq(all, 1);

		// obj
		IloNumExpr obj=cplex.numExpr();
		for (int k=0;k<noOfCause;k++){
			obj=cplex.sum(obj,cplex.prod(A[k], x[k]));
		}
		cplex.addMaximize(obj);

		if (cplex.solve()){
			if (cplex.getObjValue()>c+eps+error){
				System.out.println("obj="+cplex.getObjValue()+", c="+c);
				errorCoverCutoff2=false;
				break;
			}
		}
		if (i==ACutoff.size()-1){
			System.out.println("This is the last check! ("+ACutoff.size()+")");
		}
		//cplex.end();
	}
	cplex.end();
}


	public boolean isErrorCoverExact() {
		return errorCoverExact;
	}

	public ArrayList<int[]> getAReal() {
		return AReal;
	}

	public ArrayList<Double> getcReal() {
		return cReal;
	}

	public ArrayList<int[]> getAError() {
		return AError;
	}

	public ArrayList<Double> getcError() {
		return cError;
	}

	public void setError(double error) {
		this.error = error;
	}

	public double getObj() {
		return obj;
	}

	public ArrayList<Integer> getL0Glsit() {
		return L0Glsit;
	}

	public ArrayList<Double> getL1Glsit() {
		return L1Glsit;
	}

	public ArrayList<int[]> getACutoff() {
		return ACutoff;
	}

	public ArrayList<Double> getcCutoff() {
		return cCutoff;
	}

	public boolean isErrorCoverCutoff() {
		return errorCoverCutoff;
	}

	public boolean isErrorCoverCutoff2() {
		return errorCoverCutoff2;
	}

	public ArrayList<int[]> getACutoff2() {
		return ACutoff2;
	}

	public ArrayList<Integer> getL1List() {
		return L1List;
	}

	public ArrayList<Integer> getL0List() {
		return L0List;
	}

	public ArrayList<Integer> getL0MixL1List() {
		return L0MixL1List;
	}




}
